export class Duration {
    private seconds: number = 0;

    constructor(s: number) {
        this.seconds = s;
    }

    public static parse(s: string): Duration {
        const regex = /(\d+d)?(\d+h)?(\d+m)?([\d.]+s)?/g;
        const matches = [...s.matchAll(regex)];

        if (!matches || matches.length === 0) {
          throw new Error("Invalid duration format");
        }
      
        let totalSeconds = 0;
      
        for (const match of matches) {
          const [, days, hours, minutes, seconds] = match;
          const numericDays = parseInt(days) || 0;
          const numericHours = parseInt(hours) || 0;
          const numericMinutes = parseInt(minutes) || 0;
          const numericSeconds = parseFloat(seconds) || 0;
      
          totalSeconds +=
            numericDays * 24 * 60 * 60 +
            numericHours * 60 * 60 +
            numericMinutes * 60 +
            numericSeconds;
        }
      
        return new Duration(totalSeconds);
    }

    public toString(): string {
        let seconds = this.seconds;
        let days = Math.floor(seconds / 86400);
        seconds -= days * 86400;
        let hours = Math.floor(seconds / 3600);
        seconds -= hours * 3600;
        let minutes = Math.floor(seconds / 60);
        seconds -= minutes * 60;
        let result = "";
        if (days > 0) { result += days + "d "; }
        if (hours > 0) { result += hours + "h "; }
        if (minutes > 0) { result += minutes + "m "; }
        if (seconds > 0) { result += seconds + "s "; }
        return result;
    }

    public sprintf(format: string): string {
        let seconds = this.seconds;
        let days = Math.floor(seconds / 86400);
        seconds -= days * 86400;
        let hours = Math.floor(seconds / 3600);
        seconds -= hours * 3600;
        let minutes = Math.floor(seconds / 60);
        seconds -= minutes * 60;
        return format
            .replace("%d", days.toString())
            .replace("%h", hours.toString())
            .replace("%m", minutes.toString())
            .replace("%s", seconds.toFixed(0).toString());
    }
}
